from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from jb_drf_auth.serializers import BasicLoginSerializer, SwitchProfileSerializer
from jb_drf_auth.services.login import LoginService
from jb_drf_auth.throttling import BasicLoginIPThrottle, BasicLoginIdentityThrottle


class BasicLoginView(APIView):
    permission_classes = []
    throttle_classes = [BasicLoginIPThrottle, BasicLoginIdentityThrottle]

    def post(self, request):
        serializer = BasicLoginSerializer(data=request.data, context={"request": request})
        serializer.is_valid(raise_exception=True)
        return Response(serializer.validated_data, status=status.HTTP_200_OK)


class SwitchProfileView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        serializer = SwitchProfileSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        profile = serializer.validated_data["profile"]
        client = serializer.validated_data["client"]
        device = serializer.validated_data.get("device", None)

        response = LoginService.switch_profile(request.user, profile, client, device)
        return Response(response, status=status.HTTP_200_OK)



from jb_drf_auth.providers.base import BaseSmsProvider


class ConsoleSmsProvider(BaseSmsProvider):
    """
    Debug SMS provider that prints outgoing messages to stdout.
    """

    def send_sms(self, phone_number: str, message: str):
        print(f"[jb_drf_auth][sms][console] to={phone_number} message={message}")
        return {"provider": "console", "phone_number": phone_number, "message": message}



from rest_framework import status
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from django.utils.translation import gettext as _

from jb_drf_auth.serializers import UserSerializer, UserUpdateSerializer

@api_view(["DELETE"])
@permission_classes([IsAuthenticated])
def delete_account(request):
    if request.data.get("confirmation"):
        user = request.user
        user.delete()
        return Response(_("Cuenta eliminada correctamente."), status=status.HTTP_200_OK)

    return Response(
        _("Debe confirmar la eliminacion de la cuenta."),
        status=status.HTTP_400_BAD_REQUEST,
    )


class AccountUpdateView(APIView):
    permission_classes = [IsAuthenticated]

    def patch(self, request):
        serializer = UserUpdateSerializer(instance=request.user, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(UserSerializer(request.user).data, status=status.HTTP_200_OK)

    def put(self, request):
        serializer = UserUpdateSerializer(instance=request.user, data=request.data, partial=False)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(UserSerializer(request.user).data, status=status.HTTP_200_OK)



from jb_drf_auth.conf import get_setting
from jb_drf_auth.services.client import ClientService
from jb_drf_auth.services.email_confirmation import EmailConfirmationService
from jb_drf_auth.services.login import LoginService
from jb_drf_auth.services.me import MeService
from jb_drf_auth.services.otp import OtpService
from jb_drf_auth.services.password_reset import PasswordResetService
from jb_drf_auth.services.register import RegisterService
from jb_drf_auth.services.social_auth import SocialAuthService
from jb_drf_auth.services.tokens import TokensService
from jb_drf_auth.services.user_settings import UserSettingsService

CLIENT_CHOICES = get_setting("CLIENT_CHOICES")

__all__ = [
    "CLIENT_CHOICES",
    "ClientService",
    "EmailConfirmationService",
    "LoginService",
    "MeService",
    "OtpService",
    "PasswordResetService",
    "RegisterService",
    "SocialAuthService",
    "TokensService",
    "UserSettingsService",
]



from rest_framework import serializers
from django.utils.translation import gettext_lazy as _

from jb_drf_auth.services.password_reset import PasswordResetService


class PasswordResetRequestSerializer(serializers.Serializer):
    email = serializers.EmailField()

    def save(self):
        return PasswordResetService.send_reset_email(
            self.validated_data["email"], raise_on_fail=False
        )


class PasswordResetConfirmSerializer(serializers.Serializer):
    uid = serializers.CharField()
    token = serializers.CharField()
    new_password = serializers.CharField(write_only=True)
    new_password_confirm = serializers.CharField(write_only=True)

    def validate(self, data):
        if data["new_password"] != data["new_password_confirm"]:
            raise serializers.ValidationError(_("Las contrasenas no coinciden."))
        return data

    def save(self):
        success = PasswordResetService.reset_password(
            uidb64=self.validated_data["uid"],
            token=self.validated_data["token"],
            new_password=self.validated_data["new_password"],
        )
        if not success:
            raise serializers.ValidationError(_("Token invalido o expirado."))


class PasswordChangeSerializer(serializers.Serializer):
    old_password = serializers.CharField(write_only=True)
    new_password = serializers.CharField(write_only=True)
    new_password_confirm = serializers.CharField(write_only=True)

    def validate(self, data):
        if data["new_password"] != data["new_password_confirm"]:
            raise serializers.ValidationError(_("Las contrasenas no coinciden."))
        return data

    def save(self, **kwargs):
        user = self.context["request"].user
        success = PasswordResetService.change_password(
            user=user,
            old_password=self.validated_data["old_password"],
            new_password=self.validated_data["new_password"],
        )
        if not success:
            raise serializers.ValidationError({"old_password": _("Contrasena actual incorrecta.")})



from django.contrib.auth.tokens import default_token_generator
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode
from django.utils.translation import gettext as _
from rest_framework import serializers

from jb_drf_auth.conf import get_setting
from jb_drf_auth.utils import get_email_log_model_cls, get_email_provider, render_email_template


class EmailConfirmationService:
    @staticmethod
    def send_verification_email(user, raise_on_fail: bool = True) -> bool:
        uid = urlsafe_base64_encode(force_bytes(user.pk))
        token = default_token_generator.make_token(user)
        frontend_url = get_setting("FRONTEND_URL") or ""
        verify_url = f"{frontend_url}/verify-email/?uid={uid}&token={token}"

        subject, text_body, html_body = render_email_template(
            "email_confirmation",
            {
                "user_email": user.email,
                "verify_url": verify_url,
            },
        )

        try:
            email_log_model = get_email_log_model_cls()
        except RuntimeError as exc:
            if raise_on_fail:
                raise serializers.ValidationError(
                    {"detail": _("Configura JB_DRF_AUTH_EMAIL_LOG_MODEL para usar email.")}
                ) from exc
            return False

        provider = get_email_provider()
        try:
            provider.send_email(user.email, subject, text_body, html_body)
            email_log_model.objects.create(
                to_email=user.email,
                subject=subject,
                text_body=text_body,
                html_body=html_body,
                provider=get_setting("EMAIL_PROVIDER"),
                status="sent",
                template_name="email_confirmation",
            )
            return True
        except Exception as exc:
            email_log_model.objects.create(
                to_email=user.email,
                subject=subject,
                text_body=text_body,
                html_body=html_body,
                provider=get_setting("EMAIL_PROVIDER"),
                status="failed",
                error_message=str(exc),
                template_name="email_confirmation",
            )
            if raise_on_fail:
                raise serializers.ValidationError(
                    {"detail": _("No se pudo enviar el correo. Intenta mas tarde.")}
                ) from exc
            return False



from django.contrib.auth import get_user_model
from django.utils import timezone
from django.utils.translation import gettext as _

from jb_drf_auth.conf import get_setting
from jb_drf_auth.services.email_confirmation import EmailConfirmationService
from jb_drf_auth.utils import get_profile_model_cls


User = get_user_model()


class RegisterService:
    @staticmethod
    def register_user(
        email,
        username,
        password,
        password_confirm,
        first_name,
        last_name_1,
        last_name_2,
        birthday,
        gender,
        role,
        terms_and_conditions_accepted,
    ):
        if password != password_confirm:
            raise ValueError(_("Las contraseñas no coinciden."))

        if User.objects.filter(email=email).exists():
            raise ValueError(_("El correo electrónico ya esta en uso."))
        if username and User.objects.filter(username=username).exists():
            raise ValueError(_("El nombre de usuario ya esta en uso."))

        user = User.objects.create_user(
            email=email,
            username=username,
            password=password,
            is_active=False,
        )
        if terms_and_conditions_accepted and hasattr(user, "terms_and_conditions"):
            user.terms_and_conditions = timezone.now()
            user.save(update_fields=["terms_and_conditions"])

        profile_model = get_profile_model_cls()
        profile_model.objects.create(
            user=user,
            first_name=first_name,
            last_name_1=last_name_1,
            last_name_2=last_name_2,
            birthday=birthday,
            gender=gender,
            role=role or get_setting("DEFAULT_PROFILE_ROLE"),
            is_default=True,
        )

        email_sent = EmailConfirmationService.send_verification_email(user, raise_on_fail=False)
        return user, email_sent



import os
import unittest
from unittest.mock import patch

import django

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "jb_drf_auth.tests.settings")
django.setup()

from jb_drf_auth.services.user_settings import UserSettingsService


class DummyUser:
    def __init__(self, settings=None):
        self.settings = settings if settings is not None else {}
        self.saved_with = None

    def save(self, update_fields=None):
        self.saved_with = update_fields


class DummyQuerySet:
    def __init__(self, users):
        self._users = users

    def iterator(self):
        return iter(self._users)


class DummyManager:
    def __init__(self, users):
        self._users = users

    def all(self):
        return DummyQuerySet(self._users)


class DummyUserModel:
    objects = None


class UserSettingsServiceTests(unittest.TestCase):
    def test_set_and_remove_user_setting(self):
        user = DummyUser(settings={"a": 1})

        UserSettingsService.set_user_setting(user, "b", 2)
        self.assertEqual(user.settings["b"], 2)
        self.assertEqual(user.saved_with, ["settings"])

        UserSettingsService.remove_user_setting(user, "a")
        self.assertNotIn("a", user.settings)

    def test_set_and_remove_user_feature(self):
        user = DummyUser(settings={})

        UserSettingsService.set_user_feature(user, "reports_beta", True)
        self.assertTrue(user.settings["custom_features_available"]["reports_beta"])

        UserSettingsService.remove_user_feature(user, "reports_beta")
        self.assertNotIn("reports_beta", user.settings["custom_features_available"])

    @patch("jb_drf_auth.services.user_settings.get_user_model")
    def test_set_all_users_feature(self, mock_get_user_model):
        users = [DummyUser(settings={}), DummyUser(settings={"custom_features_available": {}})]
        DummyUserModel.objects = DummyManager(users)
        mock_get_user_model.return_value = DummyUserModel

        updated = UserSettingsService.set_all_users_feature("new_dashboard", True)
        self.assertEqual(updated, 2)
        for user in users:
            self.assertTrue(user.settings["custom_features_available"]["new_dashboard"])

    @patch("jb_drf_auth.services.user_settings.get_user_model")
    def test_remove_all_users_setting(self, mock_get_user_model):
        users = [
            DummyUser(settings={"release_channel": "stable"}),
            DummyUser(settings={}),
        ]
        DummyUserModel.objects = DummyManager(users)
        mock_get_user_model.return_value = DummyUserModel

        updated = UserSettingsService.remove_all_users_setting("release_channel")
        self.assertEqual(updated, 1)
        self.assertNotIn("release_channel", users[0].settings)



from jb_drf_auth.views.account_management import AccountUpdateView, delete_account
from jb_drf_auth.views.email_confirmation import (
    AccountConfirmEmailView,
    ResendConfirmationEmailView,
)
from jb_drf_auth.views.login import BasicLoginView, SwitchProfileView
from jb_drf_auth.views.me import MeView
from jb_drf_auth.views.otp import RequestOtpCodeView, VerifyOtpCodeView
from jb_drf_auth.views.password_reset import (
    PasswordChangeView,
    PasswordResetConfirmView,
    PasswordResetRequestView,
)
from jb_drf_auth.views.profile import ProfilePictureUpdateView, ProfileViewSet
from jb_drf_auth.views.register import RegisterView
from jb_drf_auth.views.social_auth import (
    SocialLinkView,
    SocialLoginView,
    SocialPrecheckView,
    SocialUnlinkView,
)
from jb_drf_auth.views.user_admin import CreateStaffUserView, CreateSuperUserView

__all__ = [
    "delete_account",
    "AccountUpdateView",
    "AccountConfirmEmailView",
    "ResendConfirmationEmailView",
    "BasicLoginView",
    "SwitchProfileView",
    "MeView",
    "RequestOtpCodeView",
    "VerifyOtpCodeView",
    "PasswordChangeView",
    "PasswordResetConfirmView",
    "PasswordResetRequestView",
    "ProfileViewSet",
    "ProfilePictureUpdateView",
    "RegisterView",
    "SocialLoginView",
    "SocialPrecheckView",
    "SocialLinkView",
    "SocialUnlinkView",
    "CreateStaffUserView",
    "CreateSuperUserView",
]



from jb_drf_auth.serializers.device import DevicePayloadSerializer, DeviceSerializer
from jb_drf_auth.serializers.email_confirmation import (
    EmailConfirmationSerializer,
    ResendConfirmationEmailSerializer,
)
from jb_drf_auth.serializers.login import BasicLoginSerializer, SwitchProfileSerializer
from jb_drf_auth.serializers.otp import OtpCodeRequestSerializer, OtpCodeVerifySerializer
from jb_drf_auth.serializers.password_reset import (
    PasswordChangeSerializer,
    PasswordResetConfirmSerializer,
    PasswordResetRequestSerializer,
)
from jb_drf_auth.serializers.profile import ProfilePictureUpdateSerializer, ProfileSerializer
from jb_drf_auth.serializers.register import RegisterSerializer
from jb_drf_auth.serializers.social_auth import (
    SocialLinkSerializer,
    SocialLoginSerializer,
    SocialUnlinkSerializer,
)
from jb_drf_auth.serializers.user import UserSerializer
from jb_drf_auth.serializers.user_admin import UserAdminCreateSerializer
from jb_drf_auth.serializers.user_update import UserUpdateSerializer

__all__ = [
    "DevicePayloadSerializer",
    "DeviceSerializer",
    "EmailConfirmationSerializer",
    "ResendConfirmationEmailSerializer",
    "BasicLoginSerializer",
    "SwitchProfileSerializer",
    "OtpCodeRequestSerializer",
    "OtpCodeVerifySerializer",
    "PasswordChangeSerializer",
    "PasswordResetConfirmSerializer",
    "PasswordResetRequestSerializer",
    "ProfileSerializer",
    "ProfilePictureUpdateSerializer",
    "RegisterSerializer",
    "SocialLoginSerializer",
    "SocialLinkSerializer",
    "SocialUnlinkSerializer",
    "UserSerializer",
    "UserUpdateSerializer",
    "UserAdminCreateSerializer",
]



from rest_framework import serializers
from rest_framework.exceptions import NotFound
from django.utils.translation import gettext as _

from jb_drf_auth.conf import get_setting
from jb_drf_auth.serializers.profile import ProfileSerializer
from jb_drf_auth.serializers.user import UserSerializer
from jb_drf_auth.utils import get_device_model_cls, get_profile_model_cls


class MeService:
    @staticmethod
    def _settings_payload(value):
        return value if isinstance(value, dict) else {}

    @staticmethod
    def profile_completion_required(profile):
        first_name = (getattr(profile, "first_name", "") or "").strip()
        last_name_1 = (getattr(profile, "last_name_1", "") or "").strip()
        last_name_2 = (getattr(profile, "last_name_2", "") or "").strip()
        has_first_name = bool(first_name)
        has_last_name = bool(last_name_1 or last_name_2)
        return not (has_first_name and has_last_name)

    @staticmethod
    def _profile_photo_url(profile):
        picture = getattr(profile, "picture", None)
        if not picture:
            return None
        try:
            return picture.url
        except (ValueError, AttributeError):
            return None

    @staticmethod
    def get_me_mobile(user, profile, tokens):
        response = UserSerializer(user).data
        if tokens:
            response["tokens"] = tokens
        response["active_profile"] = ProfileSerializer(profile).data
        response["user_settings"] = MeService._settings_payload(getattr(user, "settings", None))
        response["profile_settings"] = MeService._settings_payload(getattr(profile, "settings", None))
        response["profile_completion_required"] = MeService.profile_completion_required(profile)
        return response

    @staticmethod
    def get_me_web(user, profile, tokens):
        role = ["admin"]
        status = "active"

        user_payload = {
            "data": {
                "display_name": profile.display_name,
                "full_name": profile.full_name,
                "photoURL": MeService._profile_photo_url(profile),
                "email": user.email,
                "username": user.username,
                "birthday": profile.birthday,
                "shortcuts": [],
            },
            "login_redirect_url": "/home",
            "role": role,
            "status": status,
        }

        response = {
            "user": user_payload,
            "active_profile": ProfileSerializer(profile).data,
            "user_settings": MeService._settings_payload(getattr(user, "settings", None)),
            "profile_settings": MeService._settings_payload(getattr(profile, "settings", None)),
            "terms_and_conditions": getattr(user, "terms_and_conditions", None),
            "profile_completion_required": MeService.profile_completion_required(profile),
        }

        if tokens:
            response["tokens"] = tokens

        return response

    @staticmethod
    def get_me(user, client, profile_id, device_token=None):
        profile_model = get_profile_model_cls()
        try:
            profile = profile_model.objects.get(id=profile_id)
        except profile_model.DoesNotExist:
            raise NotFound(_("Perfil no encontrado."))

        if client == "web":
            return MeService.get_me_web(
                user=user,
                profile=user.get_default_profile(),
                tokens=None,
            )

        if client == "mobile":
            if get_setting("AUTH_SINGLE_SESSION_ON_MOBILE"):
                if device_token:
                    try:
                        device_model = get_device_model_cls()
                    except RuntimeError:
                        raise serializers.ValidationError(
                            {"device": _("Configura JB_DRF_AUTH_DEVICE_MODEL para validar dispositivos.")}
                        )

                    device = device_model.objects.filter(token=device_token).first()
                    if device is None:
                        raise NotFound(_("No se encontro el dispositivo con el token proporcionado."))
                else:
                    raise serializers.ValidationError(
                        {"device": _("Datos del dispositivo requeridos para cliente movil.")}
                    )

            return MeService.get_me_mobile(user=user, profile=profile, tokens=None)

        raise serializers.ValidationError({"detail": _("Parametro 'client' invalido")})



from rest_framework import serializers

from jb_drf_auth.conf import get_setting
from jb_drf_auth.serializers.device import DevicePayloadSerializer
from jb_drf_auth.services.login import LoginService


CLIENT_CHOICES = get_setting("CLIENT_CHOICES")


class BasicLoginSerializer(serializers.Serializer):
    login = serializers.CharField()
    password = serializers.CharField(write_only=True)
    client = serializers.ChoiceField(choices=CLIENT_CHOICES)
    device = DevicePayloadSerializer(write_only=True, required=False)

    def validate(self, data):
        login = data.get("login")
        password = data.get("password")
        client = data.get("client")
        device_data = data.get("device")
        return LoginService.basic_login(login, password, client, device_data)


class SwitchProfileSerializer(serializers.Serializer):
    profile = serializers.IntegerField()
    client = serializers.ChoiceField(choices=CLIENT_CHOICES)
    device = DevicePayloadSerializer(write_only=True, required=False)



import boto3

from jb_drf_auth.conf import get_setting
from jb_drf_auth.providers.base import BaseSmsProvider


class AwsSnsSmsProvider(BaseSmsProvider):
    def __init__(self):
        self.client = boto3.client("sns")

    def send_sms(self, phone_number: str, message: str):
        message_attributes = {
            "AWS.SNS.SMS.SMSType": {
                "DataType": "String",
                "StringValue": get_setting("SMS_TYPE"),
            }
        }
        sender_id = get_setting("SMS_SENDER_ID")
        if sender_id:
            message_attributes["AWS.SNS.SMS.SenderID"] = {
                "DataType": "String",
                "StringValue": sender_id,
            }

        return self.client.publish(
            PhoneNumber=phone_number,
            Message=message,
            MessageAttributes=message_attributes,
        )



import re

from django.apps import apps
from django.contrib.auth import get_user_model
from django.utils.module_loading import import_string

from .conf import get_setting, get_social_settings

def get_user_model_cls():
    return get_user_model()

def get_profile_model_cls():
    model_path = get_setting("PROFILE_MODEL")
    if not model_path:
        raise RuntimeError("Missing setting: JB_DRF_AUTH_PROFILE_MODEL = 'app_label.ModelName'")

    try:
        app_label, model_name = model_path.split(".")
    except ValueError as exc:
        raise RuntimeError(
            "Invalid JB_DRF_AUTH_PROFILE_MODEL format. Expected 'app_label.ModelName'"
        ) from exc

    return apps.get_model(app_label, model_name)


def get_device_model_cls():
    model_path = get_setting("DEVICE_MODEL")
    if not model_path:
        raise RuntimeError("Missing setting: JB_DRF_AUTH_DEVICE_MODEL = 'app_label.ModelName'")

    try:
        app_label, model_name = model_path.split(".")
    except ValueError as exc:
        raise RuntimeError(
            "Invalid JB_DRF_AUTH_DEVICE_MODEL format. Expected 'app_label.ModelName'"
        ) from exc

    return apps.get_model(app_label, model_name)


def get_otp_model_cls():
    model_path = get_setting("OTP_MODEL")
    if not model_path:
        raise RuntimeError("Missing setting: JB_DRF_AUTH_OTP_MODEL = 'app_label.ModelName'")

    try:
        app_label, model_name = model_path.split(".")
    except ValueError as exc:
        raise RuntimeError(
            "Invalid JB_DRF_AUTH_OTP_MODEL format. Expected 'app_label.ModelName'"
        ) from exc

    return apps.get_model(app_label, model_name)


def import_from_path(path: str):
    return import_string(path)


def get_sms_provider():
    provider_path = get_setting("SMS_PROVIDER")
    provider_cls = import_string(provider_path)
    return provider_cls()


def get_email_provider():
    provider_path = get_setting("EMAIL_PROVIDER")
    provider_cls = import_string(provider_path)
    return provider_cls()


def get_sms_log_model_cls():
    model_path = get_setting("SMS_LOG_MODEL")
    if not model_path:
        raise RuntimeError("Missing setting: JB_DRF_AUTH_SMS_LOG_MODEL = 'app_label.ModelName'")

    try:
        app_label, model_name = model_path.split(".")
    except ValueError as exc:
        raise RuntimeError(
            "Invalid JB_DRF_AUTH_SMS_LOG_MODEL format. Expected 'app_label.ModelName'"
        ) from exc

    return apps.get_model(app_label, model_name)


def get_email_log_model_cls():
    model_path = get_setting("EMAIL_LOG_MODEL")
    if not model_path:
        raise RuntimeError("Missing setting: JB_DRF_AUTH_EMAIL_LOG_MODEL = 'app_label.ModelName'")

    try:
        app_label, model_name = model_path.split(".")
    except ValueError as exc:
        raise RuntimeError(
            "Invalid JB_DRF_AUTH_EMAIL_LOG_MODEL format. Expected 'app_label.ModelName'"
        ) from exc

    return apps.get_model(app_label, model_name)


def get_social_account_model_cls():
    model_path = get_setting("SOCIAL_ACCOUNT_MODEL")
    if not model_path:
        raise RuntimeError("Missing setting: JB_DRF_AUTH_SOCIAL_ACCOUNT_MODEL = 'app_label.ModelName'")

    try:
        app_label, model_name = model_path.split(".")
    except ValueError as exc:
        raise RuntimeError(
            "Invalid JB_DRF_AUTH_SOCIAL_ACCOUNT_MODEL format. Expected 'app_label.ModelName'"
        ) from exc

    return apps.get_model(app_label, model_name)


def get_social_provider(provider: str):
    social_settings = get_social_settings()
    providers = social_settings.get("PROVIDERS", {})
    if not isinstance(providers, dict) or provider not in providers:
        raise RuntimeError(f"Unsupported social provider: {provider}")

    provider_cfg = providers.get(provider, {})
    if not isinstance(provider_cfg, dict):
        raise RuntimeError(f"Invalid social provider configuration for: {provider}")

    provider_path = provider_cfg.get("CLASS")
    if not provider_path:
        raise RuntimeError(f"Missing social provider class for: {provider}")

    provider_cls = import_string(provider_path)
    return provider_cls(provider=provider, provider_settings=provider_cfg)


def normalize_phone_number(raw_phone: str) -> str:
    if not raw_phone:
        return raw_phone

    phone = raw_phone.strip()
    phone = phone.replace(" ", "").replace("-", "").replace("(", "").replace(")", "")

    if phone.startswith("00"):
        phone = f"+{phone[2:]}"

    if phone.startswith("+"):
        digits = re.sub(r"\D", "", phone[1:])
        phone = f"+{digits}"
    else:
        digits = re.sub(r"\D", "", phone)
        default_cc = get_setting("PHONE_DEFAULT_COUNTRY_CODE")
        if default_cc:
            phone = f"+{default_cc}{digits}"
        else:
            raise ValueError("Phone number must include '+' and country code.")

    length = len(phone.replace("+", ""))
    min_len = get_setting("PHONE_MIN_LENGTH")
    max_len = get_setting("PHONE_MAX_LENGTH")
    if length < min_len or length > max_len:
        raise ValueError("Invalid phone number length.")

    return phone


def get_sms_message(code: str, minutes: int) -> str:
    template = get_setting("SMS_OTP_MESSAGE") or "Tu codigo es {code}. Expira en {minutes} minutos."
    message = template.format(code=code, minutes=minutes)
    if not message.isascii():
        return f"Tu codigo es {code}. Expira en {minutes} minutos."
    return message


def get_email_template(name: str):
    from jb_drf_auth.email_templates import DEFAULT_EMAIL_TEMPLATES

    templates = get_setting("EMAIL_TEMPLATES")
    if isinstance(templates, dict) and name in templates:
        return templates[name]
    return DEFAULT_EMAIL_TEMPLATES.get(name, {})


def render_email_template(name: str, context: dict):
    template = get_email_template(name)
    subject = template.get("subject", "")
    text_body = template.get("text", "")
    html_body = template.get("html")

    if callable(subject):
        subject = subject(context)
    else:
        subject = subject.format(**context)

    if callable(text_body):
        text_body = text_body(context)
    else:
        text_body = text_body.format(**context)

    if html_body is not None:
        if callable(html_body):
            html_body = html_body(context)
        else:
            html_body = html_body.format(**context)

    return subject, text_body, html_body



"""Serializers for device payloads and models."""

from rest_framework import serializers

from jb_drf_auth.utils import get_device_model_cls


class DevicePayloadSerializer(serializers.Serializer):
    platform = serializers.CharField(required=False, allow_blank=True, allow_null=True)
    name = serializers.CharField(required=False, allow_blank=True, allow_null=True)
    token = serializers.CharField(required=False, allow_blank=True, allow_null=True)
    notification_token = serializers.CharField(required=False, allow_blank=True, allow_null=True)


def _safe_exclude_fields(model, fields):
    model_fields = {field.name for field in model._meta.get_fields()}
    return tuple(field for field in fields if field in model_fields)


class DeviceSerializer(serializers.ModelSerializer):
    class Meta:
        model = None
        exclude = ()

    def get_fields(self):
        if self.Meta.model is None:
            self.Meta.model = get_device_model_cls()
            self.Meta.exclude = _safe_exclude_fields(
                self.Meta.model, ("deleted", "deleted_by_cascade", "user")
            )
        return super().get_fields()



"""Register serializer."""

from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

from jb_drf_auth.conf import get_setting
from jb_drf_auth.services.register import RegisterService


class RegisterSerializer(serializers.Serializer):
    email = serializers.EmailField()
    username = serializers.CharField(required=False, allow_blank=True, allow_null=True)
    password = serializers.CharField(min_length=8, max_length=128)
    password_confirm = serializers.CharField(min_length=8, max_length=128)
    first_name = serializers.CharField(max_length=100)
    last_name_1 = serializers.CharField(max_length=150)
    last_name_2 = serializers.CharField(max_length=150, required=False, allow_blank=True, allow_null=True)
    birthday = serializers.DateField(required=True)
    gender = serializers.CharField(max_length=50)
    role = serializers.CharField(required=False, allow_blank=True, allow_null=True)
    terms_and_conditions_accepted = serializers.BooleanField(required=False, default=False)

    def validate(self, attrs):
        if get_setting("TERMS_AND_CONDITIONS_REQUIRED") and not attrs.get(
            "terms_and_conditions_accepted", False
        ):
            raise serializers.ValidationError(
                {"terms_and_conditions_accepted": _("Debes aceptar terminos y condiciones.")}
            )
        return attrs

    def create(self, validated_data):
        user, email_sent = RegisterService.register_user(
            email=validated_data["email"],
            username=validated_data.get("username"),
            password=validated_data["password"],
            password_confirm=validated_data["password_confirm"],
            first_name=validated_data["first_name"],
            last_name_1=validated_data["last_name_1"],
            last_name_2=validated_data.get("last_name_2"),
            birthday=validated_data["birthday"],
            gender=validated_data["gender"],
            role=validated_data.get("role"),
            terms_and_conditions_accepted=validated_data.get(
                "terms_and_conditions_accepted", False
            ),
        )
        self.email_sent = email_sent
        return user



from django.apps import AppConfig

class JbDrfAuthConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "jb_drf_auth"
    verbose_name = "JB DRF Auth"

    def ready(self):
        from jb_drf_auth import checks  # noqa: F401



import os
import unittest
from types import SimpleNamespace
from unittest.mock import MagicMock, patch

import django

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "jb_drf_auth.tests.settings")
django.setup()

from jb_drf_auth.services.password_reset import PasswordResetService


class PasswordResetServiceTests(unittest.TestCase):
    @patch("jb_drf_auth.services.password_reset.get_setting")
    @patch("jb_drf_auth.services.password_reset.get_email_provider")
    @patch("jb_drf_auth.services.password_reset.get_email_log_model_cls")
    @patch("jb_drf_auth.services.password_reset.render_email_template")
    @patch("jb_drf_auth.services.password_reset.User")
    def test_send_reset_email_sent_and_logged(
        self,
        user_cls,
        render_email_template,
        get_email_log_model_cls,
        get_email_provider,
        get_setting,
    ):
        user = SimpleNamespace(
            pk=1,
            email="user@example.com",
            password="pbkdf2_sha256$260000$mock$hash",
            last_login=None,
        )
        user.get_email_field_name = lambda: "email"
        user_cls.objects.get.return_value = user
        render_email_template.return_value = ("Reset", "text body", "<p>html body</p>")
        get_setting.side_effect = lambda key: {
            "EMAIL_PROVIDER": "jb_drf_auth.providers.console_email.ConsoleEmailProvider",
            "FRONTEND_URL": "http://localhost:3000",
        }.get(key)

        email_log_model = MagicMock()
        get_email_log_model_cls.return_value = email_log_model
        provider = MagicMock()
        get_email_provider.return_value = provider

        sent = PasswordResetService.send_reset_email("user@example.com", raise_on_fail=False)

        self.assertEqual(sent, True)
        provider.send_email.assert_called_once()
        email_log_model.objects.create.assert_called_once()
        self.assertEqual(email_log_model.objects.create.call_args.kwargs["status"], "sent")

    @patch("jb_drf_auth.services.password_reset.get_setting")
    @patch("jb_drf_auth.services.password_reset.get_email_log_model_cls")
    @patch("jb_drf_auth.services.password_reset.User")
    def test_send_reset_email_user_not_found_logs_failed(
        self,
        user_cls,
        get_email_log_model_cls,
        get_setting,
    ):
        does_not_exist = type("DoesNotExist", (Exception,), {})
        user_cls.DoesNotExist = does_not_exist
        user_cls.objects.get.side_effect = does_not_exist
        get_setting.side_effect = lambda key: {
            "EMAIL_PROVIDER": "jb_drf_auth.providers.console_email.ConsoleEmailProvider",
        }.get(key)

        email_log_model = MagicMock()
        get_email_log_model_cls.return_value = email_log_model

        sent = PasswordResetService.send_reset_email("missing@example.com", raise_on_fail=False)

        self.assertEqual(sent, False)
        email_log_model.objects.create.assert_called_once()
        kwargs = email_log_model.objects.create.call_args.kwargs
        self.assertEqual(kwargs["to_email"], "missing@example.com")
        self.assertEqual(kwargs["status"], "failed")
        self.assertEqual(kwargs["error_message"], "user_not_found")



from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers


class UserUpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = get_user_model()
        fields = ()

    def get_fields(self):
        allowed = ("email", "username", "phone", "terms_and_conditions")
        model_field_names = {field.name for field in self.Meta.model._meta.get_fields()}
        self.Meta.fields = tuple(field for field in allowed if field in model_field_names)
        fields = super().get_fields()
        fields["language"] = serializers.CharField(required=False, allow_blank=False)
        fields["timezone"] = serializers.CharField(required=False, allow_blank=False)
        return fields

    def validate_email(self, value):
        user_model = self.Meta.model
        instance = self.instance
        if value and user_model.objects.filter(email=value).exclude(pk=instance.pk).exists():
            raise serializers.ValidationError(_("Ya existe un usuario con este correo."))
        return value

    def validate_username(self, value):
        user_model = self.Meta.model
        instance = self.instance
        if value and user_model.objects.filter(username=value).exclude(pk=instance.pk).exists():
            raise serializers.ValidationError(_("El nombre de usuario ya esta en uso."))
        return value

    def validate_phone(self, value):
        user_model = self.Meta.model
        instance = self.instance
        if value and user_model.objects.filter(phone=value).exclude(pk=instance.pk).exists():
            raise serializers.ValidationError(_("El telefono ya esta en uso."))
        return value

    def update(self, instance, validated_data):
        language = validated_data.pop("language", None)
        timezone = validated_data.pop("timezone", None)

        instance = super().update(instance, validated_data)

        if language is not None or timezone is not None:
            payload = instance.settings if isinstance(instance.settings, dict) else {}
            if language is not None:
                payload["language"] = language
            if timezone is not None:
                payload["timezone"] = timezone
            instance.settings = payload
            instance.save(update_fields=["settings"])

        return instance



from rest_framework import serializers
from django.utils.translation import gettext as _

from jb_drf_auth.conf import get_setting
from jb_drf_auth.services.me import MeService
from jb_drf_auth.utils import get_device_model_cls


class ClientService:
    @staticmethod
    def response_for_client(client, user, profile, tokens, device_data):
        if client.lower() == "mobile":
            if not device_data:
                raise serializers.ValidationError(
                    {"device": _("Datos del dispositivo requeridos para cliente movil.")}
                )

            try:
                device_model = get_device_model_cls()
            except RuntimeError:
                raise serializers.ValidationError(
                    {"device": _("Configura JB_DRF_AUTH_DEVICE_MODEL para registrar dispositivos.")}
                )

            notification_token = device_data.get("notification_token")
            require_notification_token = bool(get_setting("MOBILE_NOTIFICATION_TOKEN_REQUIRED"))
            if require_notification_token and not notification_token:
                raise serializers.ValidationError(
                    {"device": _("notification_token es requerido para cliente movil.")}
                )

            token = device_data.get("token")
            if token:
                device_model.objects.update_or_create(
                    user=user,
                    token=token,
                    defaults={
                        "platform": device_data.get("platform", "Unknown Platform"),
                        "name": device_data.get("name", "Unknown Device"),
                        "notification_token": notification_token,
                    },
                )
            else:
                device_model.objects.create(
                    user=user,
                    platform=device_data.get("platform", "Unknown Platform"),
                    name=device_data.get("name", "Unknown Device"),
                    token=None,
                    notification_token=notification_token,
                )

            response_data = MeService.get_me_mobile(user, profile, tokens)
            response_data["device_registered"] = True
            return response_data

        return MeService.get_me_web(user, profile, tokens)



import secrets

from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from rest_framework import permissions

from jb_drf_auth.conf import get_setting


class BootstrapTokenOrAdmin(permissions.BasePermission):
    message = _("Bootstrap token required.")

    def has_permission(self, request, view):
        user = request.user
        if user and user.is_authenticated and user.is_superuser:
            return True

        user_model = get_user_model()
        if user_model.objects.filter(is_superuser=True).exists():
            return False

        token = request.headers.get("X-Admin-Bootstrap-Token")
        bootstrap_token = get_setting("ADMIN_BOOTSTRAP_TOKEN") or ""
        if not bootstrap_token or not token:
            return False

        return secrets.compare_digest(token, bootstrap_token)



import json
from urllib.error import HTTPError, URLError
from urllib.parse import urlencode
from urllib.request import Request, urlopen

from jb_drf_auth.conf import get_setting
from jb_drf_auth.providers.base import BaseSmsProvider


class TwilioSmsProvider(BaseSmsProvider):
    def send_sms(self, phone_number: str, message: str):
        account_sid = get_setting("TWILIO_ACCOUNT_SID")
        auth_token = get_setting("TWILIO_AUTH_TOKEN")
        from_number = get_setting("TWILIO_FROM_NUMBER")
        messaging_service_sid = get_setting("TWILIO_MESSAGING_SERVICE_SID")

        if not account_sid or not auth_token:
            raise RuntimeError("Twilio account credentials are not configured.")
        if not from_number and not messaging_service_sid:
            raise RuntimeError(
                "Configure TWILIO_FROM_NUMBER or TWILIO_MESSAGING_SERVICE_SID."
            )

        payload = {
            "To": phone_number,
            "Body": message,
        }
        if messaging_service_sid:
            payload["MessagingServiceSid"] = messaging_service_sid
        else:
            payload["From"] = from_number

        endpoint = f"https://api.twilio.com/2010-04-01/Accounts/{account_sid}/Messages.json"
        req = Request(
            endpoint,
            data=urlencode(payload).encode("utf-8"),
            method="POST",
            headers={"Content-Type": "application/x-www-form-urlencoded"},
        )
        req.add_header("Authorization", f"Basic {self._build_basic_auth(account_sid, auth_token)}")
        try:
            with urlopen(req, timeout=10) as response:
                body = response.read().decode("utf-8")
                return json.loads(body) if body else {"status": response.status}
        except HTTPError as exc:
            error_payload = exc.read().decode("utf-8", errors="ignore")
            raise RuntimeError(f"Twilio delivery failed: {error_payload}") from exc
        except (URLError, TimeoutError) as exc:
            raise RuntimeError("Twilio delivery failed: network error.") from exc

    @staticmethod
    def _build_basic_auth(username: str, password: str) -> str:
        import base64

        raw = f"{username}:{password}".encode("utf-8")
        return base64.b64encode(raw).decode("ascii")



from rest_framework import status
from rest_framework.exceptions import AuthenticationFailed
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from django.utils.translation import gettext as _

from jb_drf_auth.services.me import MeService


class MeView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        user = request.user
        auth_payload = request.auth or {}
        profile_id = auth_payload.get("profile_id")
        client = request.query_params.get("client")
        device_token = request.query_params.get("device_token")

        if not profile_id:
            raise AuthenticationFailed(_("Falta perfil en el token"))

        if getattr(user, "deleted", None) or not getattr(user, "is_active", True) or not getattr(
            user, "is_verified", True
        ):
            return Response(
                {"detail": _("Usuario no valido, eliminado, inactivo o no verificado")},
                status=status.HTTP_401_UNAUTHORIZED,
            )

        response = MeService.get_me(
            user=user,
            client=client,
            profile_id=profile_id,
            device_token=device_token,
        )

        return Response(response, status=status.HTTP_200_OK)



import uuid
import logging
from urllib.error import URLError
from urllib.request import urlopen

from django.contrib.auth import get_user_model
from django.core.files.base import ContentFile
from django.utils import timezone
from django.utils.translation import gettext as _
from rest_framework import status

from jb_drf_auth.conf import get_setting, get_social_settings
from jb_drf_auth.exceptions import SocialAuthError
from jb_drf_auth.services.client import ClientService
from jb_drf_auth.services.tokens import TokensService
from jb_drf_auth.utils import (
    get_profile_model_cls,
    get_social_account_model_cls,
    get_social_provider,
)


User = get_user_model()
logger = logging.getLogger("jb_drf_auth.services.social_auth")


class SocialAuthService:
    @staticmethod
    def _normalize_username(base: str) -> str:
        value = (base or "social").strip().lower().replace(" ", "_")
        if not value:
            value = "social"
        return value[:140]

    @staticmethod
    def _build_unique_username(provider: str, provider_user_id: str, email: str | None) -> str:
        if email and "@" in email:
            base = SocialAuthService._normalize_username(email.split("@")[0])
        else:
            base = SocialAuthService._normalize_username(f"{provider}_{provider_user_id}")

        candidate = base
        counter = 1
        while User.objects.filter(username=candidate).exists():
            counter += 1
            suffix = f"_{counter}"
            candidate = f"{base[: max(1, 150 - len(suffix))]}{suffix}"
        return candidate

    @staticmethod
    def _create_user_from_identity(identity, terms_accepted: bool, role: str | None = None):
        social_settings = get_social_settings()
        require_verified = social_settings.get("REQUIRE_VERIFIED_EMAIL", True)
        if require_verified and not identity.email_verified:
            raise SocialAuthError(
                _("Social provider did not return a verified email."),
                status_code=status.HTTP_400_BAD_REQUEST,
                code="social_email_not_verified",
            )

        if not identity.email:
            raise SocialAuthError(
                _("Social provider did not return a valid email."),
                status_code=status.HTTP_400_BAD_REQUEST,
                code="social_email_missing",
            )

        if get_setting("TERMS_AND_CONDITIONS_REQUIRED") and not terms_accepted:
            raise SocialAuthError(
                _("Terms and conditions must be accepted."),
                status_code=status.HTTP_400_BAD_REQUEST,
                code="terms_required",
            )

        username = SocialAuthService._build_unique_username(
            identity.provider, identity.provider_user_id, identity.email
        )
        user = User.objects.create_user(
            email=identity.email,
            username=username,
            password=None,
            is_active=True,
        )
        user.set_unusable_password()

        update_fields = []
        if hasattr(user, "is_verified"):
            user.is_verified = bool(identity.email_verified)
            update_fields.append("is_verified")
        if terms_accepted and hasattr(user, "terms_and_conditions"):
            user.terms_and_conditions = timezone.now()
            update_fields.append("terms_and_conditions")
        if update_fields:
            update_fields.append("password")
            user.save(update_fields=update_fields)
        else:
            user.save()

        profile_model = get_profile_model_cls()
        profile_model.objects.create(
            user=user,
            first_name=identity.first_name,
            last_name_1=identity.last_name_1,
            role=role or get_setting("DEFAULT_PROFILE_ROLE"),
            is_default=True,
        )
        logger.info(
            "social_user_created provider=%s user_id=%s has_email=%s",
            identity.provider,
            getattr(user, "id", None),
            bool(identity.email),
        )
        return user

    @staticmethod
    def _sync_profile_picture(profile, picture_url: str | None):
        social_settings = get_social_settings()
        if not social_settings.get("SYNC_PICTURE_ON_LOGIN", True):
            return
        if not picture_url or not hasattr(profile, "picture"):
            return

        timeout = social_settings.get("PICTURE_DOWNLOAD_TIMEOUT_SECONDS", 5)
        max_bytes = int(social_settings.get("PICTURE_MAX_BYTES", 5 * 1024 * 1024))
        allowed_types = social_settings.get("PICTURE_ALLOWED_CONTENT_TYPES") or ()
        allowed_types = {str(value).lower() for value in allowed_types}
        try:
            with urlopen(picture_url, timeout=timeout) as response:
                content_type = response.headers.get("Content-Type", "").lower()
                content_type = content_type.split(";")[0].strip()
                if not content_type.startswith("image/"):
                    return
                if allowed_types and content_type not in allowed_types:
                    return
                payload = response.read(max_bytes + 1)
        except (URLError, TimeoutError, ValueError, OSError):
            logger.warning(
                "social_picture_download_failed profile_id=%s picture_url_present=%s",
                getattr(profile, "pk", None),
                bool(picture_url),
            )
            return

        if not payload:
            return
        if len(payload) > max_bytes:
            return

        extension = "jpg"
        if "png" in content_type:
            extension = "png"
        elif "webp" in content_type:
            extension = "webp"

        filename = f"social-{profile.pk}-{uuid.uuid4().hex[:10]}.{extension}"
        profile.picture.save(filename, ContentFile(payload), save=True)
        logger.info(
            "social_picture_synced profile_id=%s bytes=%s content_type=%s",
            getattr(profile, "pk", None),
            len(payload),
            content_type,
        )

    @staticmethod
    def login_or_register(
        provider_name: str,
        payload: dict,
        client: str,
        device_data: dict | None,
        role: str | None = None,
        terms_and_conditions_accepted: bool = False,
    ):
        logger.info(
            "social_login_or_register_started provider=%s client=%s",
            provider_name,
            client,
        )
        social_provider = get_social_provider(provider_name)
        identity = social_provider.authenticate(payload)

        social_account_model = get_social_account_model_cls()
        social_settings = get_social_settings()
        user = None
        user_created = False
        linked_existing = False

        social_account = social_account_model.objects.filter(
            provider=identity.provider,
            provider_user_id=identity.provider_user_id,
        ).select_related("user").first()

        if social_account:
            user = social_account.user
            logger.info(
                "social_account_found provider=%s user_id=%s",
                identity.provider,
                getattr(user, "id", None),
            )
        else:
            if social_settings.get("LINK_BY_EMAIL", True) and identity.email:
                user = User.objects.filter(email__iexact=identity.email).first()
                linked_existing = user is not None
                if linked_existing:
                    logger.info(
                        "social_linked_existing_by_email provider=%s user_id=%s",
                        identity.provider,
                        getattr(user, "id", None),
                    )

            if user is None:
                if not social_settings.get("AUTO_CREATE_USER", True):
                    raise SocialAuthError(
                        _("No account is linked for this social provider."),
                        status_code=status.HTTP_400_BAD_REQUEST,
                        code="social_account_not_linked",
                    )
                user = SocialAuthService._create_user_from_identity(
                    identity=identity,
                    terms_accepted=terms_and_conditions_accepted,
                    role=role,
                )
                user_created = True

        social_account, created_or_updated = social_account_model.objects.update_or_create(
            provider=identity.provider,
            provider_user_id=identity.provider_user_id,
            defaults={
                "user": user,
                "email": identity.email,
                "email_verified": bool(identity.email_verified),
                "picture_url": identity.picture_url,
                "raw_response": identity.raw_response or {},
                "last_login_at": timezone.now(),
            },
        )

        profile = user.get_default_profile()
        if profile is None:
            profile_model = get_profile_model_cls()
            profile = profile_model.objects.create(
                user=user,
                first_name=identity.first_name,
                last_name_1=identity.last_name_1,
                role=role or get_setting("DEFAULT_PROFILE_ROLE"),
                is_default=True,
            )

        SocialAuthService._sync_profile_picture(profile, identity.picture_url)

        tokens = TokensService.get_tokens_for_user(
            user=user,
            profile=profile,
        )
        response = ClientService.response_for_client(client, user, profile, tokens, device_data)
        response["social_provider"] = identity.provider
        response["user_created"] = user_created
        response["linked_existing_user"] = linked_existing
        response["social_account_id"] = social_account.pk
        logger.info(
            "social_login_or_register_success provider=%s user_id=%s user_created=%s linked_existing_user=%s",
            identity.provider,
            getattr(user, "id", None),
            user_created,
            linked_existing,
        )
        return response

    @staticmethod
    def precheck(provider_name: str, payload: dict):
        social_provider = get_social_provider(provider_name)
        identity = social_provider.authenticate(payload)

        social_account_model = get_social_account_model_cls()
        social_settings = get_social_settings()
        link_by_email = bool(social_settings.get("LINK_BY_EMAIL", True))
        auto_create_user = bool(social_settings.get("AUTO_CREATE_USER", True))

        social_account = social_account_model.objects.filter(
            provider=identity.provider,
            provider_user_id=identity.provider_user_id,
        ).select_related("user").first()

        user_by_email = None
        if not social_account and link_by_email and identity.email:
            user_by_email = User.objects.filter(email__iexact=identity.email).first()

        social_account_exists = social_account is not None
        linked_existing_user = user_by_email is not None
        user_exists = social_account_exists or linked_existing_user
        would_create_user = (not user_exists) and auto_create_user

        return {
            "provider": identity.provider,
            "email": identity.email,
            "email_verified": bool(identity.email_verified),
            "social_account_exists": social_account_exists,
            "linked_existing_user": linked_existing_user,
            "user_exists": user_exists,
            "would_create_user": would_create_user,
            "can_login": social_account_exists or linked_existing_user or auto_create_user,
        }

    @staticmethod
    def link_account(user, provider_name: str, payload: dict):
        logger.info(
            "social_link_started provider=%s user_id=%s",
            provider_name,
            getattr(user, "id", None),
        )
        social_provider = get_social_provider(provider_name)
        identity = social_provider.authenticate(payload)

        social_account_model = get_social_account_model_cls()

        existing = social_account_model.objects.filter(
            provider=identity.provider,
            provider_user_id=identity.provider_user_id,
        ).select_related("user").first()
        if existing and existing.user_id != user.id:
            logger.warning(
                "social_link_rejected provider=%s target_user_id=%s existing_user_id=%s",
                identity.provider,
                getattr(user, "id", None),
                existing.user_id,
            )
            raise SocialAuthError(
                _("This social account is already linked to another user."),
                status_code=status.HTTP_400_BAD_REQUEST,
                code="social_already_linked",
            )

        social_account, created = social_account_model.objects.update_or_create(
            provider=identity.provider,
            provider_user_id=identity.provider_user_id,
            defaults={
                "user": user,
                "email": identity.email,
                "email_verified": bool(identity.email_verified),
                "picture_url": identity.picture_url,
                "raw_response": identity.raw_response or {},
                "last_login_at": timezone.now(),
            },
        )

        profile = user.get_default_profile()
        if profile is not None:
            SocialAuthService._sync_profile_picture(profile, identity.picture_url)

        logger.info(
            "social_link_success provider=%s user_id=%s created=%s",
            identity.provider,
            getattr(user, "id", None),
            created,
        )
        return {
            "detail": _("Social account linked successfully."),
            "provider": identity.provider,
            "social_account_id": social_account.pk,
            "created": created,
        }

    @staticmethod
    def unlink_account(user, provider_name: str):
        logger.info(
            "social_unlink_started provider=%s user_id=%s",
            provider_name,
            getattr(user, "id", None),
        )
        social_account_model = get_social_account_model_cls()
        social_account = social_account_model.objects.filter(
            user=user,
            provider=provider_name,
        ).first()
        if not social_account:
            logger.warning(
                "social_unlink_not_found provider=%s user_id=%s",
                provider_name,
                getattr(user, "id", None),
            )
            raise SocialAuthError(
                _("No linked social account found for this provider."),
                status_code=status.HTTP_404_NOT_FOUND,
                code="social_not_found",
            )

        social_account.delete()
        logger.info(
            "social_unlink_success provider=%s user_id=%s",
            provider_name,
            getattr(user, "id", None),
        )
        return {
            "detail": _("Social account unlinked successfully."),
            "provider": provider_name,
        }



from jb_drf_auth.providers.base import BaseEmailProvider


class ConsoleEmailProvider(BaseEmailProvider):
    """
    Debug email provider that prints outgoing messages to stdout.
    """

    def send_email(self, to_email: str, subject: str, text_body: str, html_body: str | None = None):
        print(
            "[jb_drf_auth][email][console] "
            f"to={to_email} subject={subject} text_body={text_body} html_body={html_body}"
        )
        return {
            "provider": "console",
            "to_email": to_email,
            "subject": subject,
            "text_body": text_body,
            "html_body": html_body,
        }



from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from jb_drf_auth.serializers import OtpCodeRequestSerializer, OtpCodeVerifySerializer
from jb_drf_auth.services.otp import OtpService
from jb_drf_auth.throttling import (
    OtpRequestIPThrottle,
    OtpRequestIdentityThrottle,
    OtpVerifyIPThrottle,
    OtpVerifyIdentityThrottle,
)


class RequestOtpCodeView(APIView):
    throttle_classes = [OtpRequestIPThrottle, OtpRequestIdentityThrottle]

    def post(self, request):
        serializer = OtpCodeRequestSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        response = OtpService.request_otp_code(data)
        return Response(response, status=status.HTTP_201_CREATED)


class VerifyOtpCodeView(APIView):
    throttle_classes = [OtpVerifyIPThrottle, OtpVerifyIdentityThrottle]

    def post(self, request):
        serializer = OtpCodeVerifySerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        response = OtpService.verify_otp_code(data)
        return Response(response, status=status.HTTP_200_OK)



from rest_framework_simplejwt.tokens import RefreshToken

from jb_drf_auth.conf import get_setting


class TokensService:
    @staticmethod
    def get_tokens_for_user(user, profile):
        if not profile:
            raise ValueError(
                "Se debe proporcionar un perfil valido para el usuario para generar tokens."
            )

        refresh = RefreshToken.for_user(user)
        refresh[get_setting("PROFILE_ID_CLAIM")] = profile.id
        return {
            "refreshToken": str(refresh),
            "accessToken": str(refresh.access_token),
        }



DEFAULT_EMAIL_TEMPLATES = {
    "email_confirmation": {
        "subject": "Verifica tu correo",
        "text": (
            "Hola {user_email},\n\n"
            "Verifica tu correo usando este enlace:\n"
            "{verify_url}\n\n"
            "Si no solicitaste esta acción, ignora este mensaje."
        ),
        "html": (
            "<!doctype html>"
            "<html>"
            "<body style=\"margin:0;padding:0;background:#f7f7f7;font-family:Arial,sans-serif;\">"
            "<div style=\"max-width:560px;margin:24px auto;background:#ffffff;border-radius:12px;"
            "padding:24px;box-shadow:0 8px 24px rgba(0,0,0,.08);\">"
            "<h2 style=\"margin:0 0 12px 0;color:#111;\">Verifica tu correo</h2>"
            "<p style=\"margin:0 0 16px 0;color:#444;\">Hola {user_email},</p>"
            "<p style=\"margin:0 0 20px 0;color:#444;\">"
            "Confirma tu cuenta haciendo clic en el botón:"
            "</p>"
            "<a href=\"{verify_url}\" "
            "style=\"display:inline-block;padding:12px 18px;background:#111;color:#fff;"
            "text-decoration:none;border-radius:8px;\">Verificar correo</a>"
            "<p style=\"margin:20px 0 0 0;color:#777;font-size:12px;\">"
            "Si no solicitaste esta acción, ignora este mensaje."
            "</p>"
            "</div>"
            "</body>"
            "</html>"
        ),
    },
    "password_reset": {
        "subject": "Restablece tu contraseña",
        "text": (
            "Hola {user_email},\n\n"
            "Restablece tu contraseña usando este enlace:\n"
            "{reset_url}\n\n"
            "Si no solicitaste este cambio, ignora este mensaje."
        ),
        "html": (
            "<!doctype html>"
            "<html>"
            "<body style=\"margin:0;padding:0;background:#f7f7f7;font-family:Arial,sans-serif;\">"
            "<div style=\"max-width:560px;margin:24px auto;background:#ffffff;border-radius:12px;"
            "padding:24px;box-shadow:0 8px 24px rgba(0,0,0,.08);\">"
            "<h2 style=\"margin:0 0 12px 0;color:#111;\">Restablece tu contraseña</h2>"
            "<p style=\"margin:0 0 16px 0;color:#444;\">Hola {user_email},</p>"
            "<p style=\"margin:0 0 20px 0;color:#444;\">"
            "Haz clic en el botón para cambiar tu contraseña:"
            "</p>"
            "<a href=\"{reset_url}\" "
            "style=\"display:inline-block;padding:12px 18px;background:#111;color:#fff;"
            "text-decoration:none;border-radius:8px;\">Restablecer</a>"
            "<p style=\"margin:20px 0 0 0;color:#777;font-size:12px;\">"
            "Si no solicitaste este cambio, ignora este mensaje."
            "</p>"
            "</div>"
            "</body>"
            "</html>"
        ),
    },
}



from rest_framework import serializers
from django.utils.translation import gettext_lazy as _

from jb_drf_auth.conf import get_setting, get_social_settings
from jb_drf_auth.serializers.device import DevicePayloadSerializer
from jb_drf_auth.services.social_auth import SocialAuthService


CLIENT_CHOICES = get_setting("CLIENT_CHOICES")


class SocialLoginSerializer(serializers.Serializer):
    provider = serializers.CharField()
    id_token = serializers.CharField(required=False, allow_blank=True)
    access_token = serializers.CharField(required=False, allow_blank=True)
    authorization_code = serializers.CharField(required=False, allow_blank=True)
    redirect_uri = serializers.CharField(required=False, allow_blank=True)
    code_verifier = serializers.CharField(required=False, allow_blank=True)
    client_id = serializers.CharField(required=False, allow_blank=True)
    client = serializers.ChoiceField(choices=CLIENT_CHOICES)
    device = DevicePayloadSerializer(write_only=True, required=False)
    role = serializers.CharField(required=False, allow_blank=True, allow_null=True)
    terms_and_conditions_accepted = serializers.BooleanField(required=False, default=False)

    def validate_provider(self, value):
        providers = get_social_settings().get("PROVIDERS", {})
        if value not in providers:
            raise serializers.ValidationError(_("Unsupported social provider."))
        return value

    def validate(self, data):
        provider = data.get("provider")
        if provider in ("google", "apple"):
            if not data.get("id_token") and not data.get("authorization_code"):
                raise serializers.ValidationError(
                    {"detail": _("id_token or authorization_code is required for this provider.")}
                )
            if data.get("authorization_code"):
                if not data.get("redirect_uri"):
                    raise serializers.ValidationError(
                        {"detail": _("redirect_uri is required for authorization_code flow.")}
                    )
                if not data.get("client_id"):
                    raise serializers.ValidationError(
                        {"detail": _("client_id is required for authorization_code flow.")}
                    )
        if provider == "facebook" and not data.get("access_token"):
            raise serializers.ValidationError({"detail": _("access_token is required for Facebook.")})
        return data

    def save(self, **kwargs):
        payload = {
            "id_token": self.validated_data.get("id_token"),
            "access_token": self.validated_data.get("access_token"),
            "authorization_code": self.validated_data.get("authorization_code"),
            "redirect_uri": self.validated_data.get("redirect_uri"),
            "code_verifier": self.validated_data.get("code_verifier"),
            "client_id": self.validated_data.get("client_id"),
        }
        return SocialAuthService.login_or_register(
            provider_name=self.validated_data.get("provider"),
            payload=payload,
            client=self.validated_data.get("client"),
            device_data=self.validated_data.get("device"),
            role=self.validated_data.get("role"),
            terms_and_conditions_accepted=self.validated_data.get(
                "terms_and_conditions_accepted", False
            ),
        )


class SocialLinkSerializer(serializers.Serializer):
    provider = serializers.CharField()
    id_token = serializers.CharField(required=False, allow_blank=True)
    access_token = serializers.CharField(required=False, allow_blank=True)
    authorization_code = serializers.CharField(required=False, allow_blank=True)
    redirect_uri = serializers.CharField(required=False, allow_blank=True)
    code_verifier = serializers.CharField(required=False, allow_blank=True)
    client_id = serializers.CharField(required=False, allow_blank=True)

    def validate_provider(self, value):
        providers = get_social_settings().get("PROVIDERS", {})
        if value not in providers:
            raise serializers.ValidationError(_("Unsupported social provider."))
        return value

    def validate(self, data):
        provider = data.get("provider")
        if provider in ("google", "apple"):
            if not data.get("id_token") and not data.get("authorization_code"):
                raise serializers.ValidationError(
                    {"detail": _("id_token or authorization_code is required for this provider.")}
                )
            if data.get("authorization_code"):
                if not data.get("redirect_uri"):
                    raise serializers.ValidationError(
                        {"detail": _("redirect_uri is required for authorization_code flow.")}
                    )
                if not data.get("client_id"):
                    raise serializers.ValidationError(
                        {"detail": _("client_id is required for authorization_code flow.")}
                    )
        if provider == "facebook" and not data.get("access_token"):
            raise serializers.ValidationError({"detail": _("access_token is required for Facebook.")})
        return data

    def save(self, **kwargs):
        payload = {
            "id_token": self.validated_data.get("id_token"),
            "access_token": self.validated_data.get("access_token"),
            "authorization_code": self.validated_data.get("authorization_code"),
            "redirect_uri": self.validated_data.get("redirect_uri"),
            "code_verifier": self.validated_data.get("code_verifier"),
            "client_id": self.validated_data.get("client_id"),
        }
        return SocialAuthService.link_account(
            user=self.context["request"].user,
            provider_name=self.validated_data.get("provider"),
            payload=payload,
        )


class SocialUnlinkSerializer(serializers.Serializer):
    provider = serializers.CharField()

    def validate_provider(self, value):
        providers = get_social_settings().get("PROVIDERS", {})
        if value not in providers:
            raise serializers.ValidationError(_("Unsupported social provider."))
        return value

    def save(self, **kwargs):
        return SocialAuthService.unlink_account(
            user=self.context["request"].user,
            provider_name=self.validated_data.get("provider"),
        )



import os
import unittest
from types import SimpleNamespace
from unittest.mock import MagicMock, patch

import django

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "jb_drf_auth.tests.settings")
django.setup()

from jb_drf_auth.services.otp import OtpService


class OtpServiceTests(unittest.TestCase):
    @patch("jb_drf_auth.services.otp.get_setting")
    @patch("jb_drf_auth.services.otp.get_otp_model_cls")
    @patch("jb_drf_auth.services.otp.User")
    def test_request_otp_code_returns_user_exist_flag(
        self,
        user_cls,
        get_otp_model_cls,
        get_setting,
    ):
        get_setting.side_effect = lambda key: {
            "OTP_LENGTH": 6,
            "OTP_RESEND_COOLDOWN_SECONDS": 60,
            "OTP_TTL_SECONDS": 300,
        }[key]

        latest_qs = MagicMock()
        latest_qs.order_by.return_value.first.return_value = None
        latest_qs.filter.return_value = latest_qs

        otp_model = MagicMock()
        otp_model.objects.filter.return_value = latest_qs
        otp_model.objects.create.return_value = SimpleNamespace(channel="email")
        get_otp_model_cls.return_value = otp_model

        user_cls.objects.filter.return_value.exists.return_value = True

        result = OtpService.request_otp_code(
            {
                "channel": "email",
                "email": "user@example.com",
            }
        )

        self.assertEqual(result["user_exist"], True)
        self.assertEqual(result["channel"], "email")

    @patch("jb_drf_auth.services.otp.get_setting")
    @patch("jb_drf_auth.services.otp.ClientService.response_for_client")
    @patch("jb_drf_auth.services.otp.TokensService.get_tokens_for_user")
    @patch("jb_drf_auth.services.otp.get_profile_model_cls")
    @patch("jb_drf_auth.services.otp.get_otp_model_cls")
    @patch("jb_drf_auth.services.otp.User")
    def test_verify_otp_code_phone_only_uses_fallback_email(
        self,
        user_cls,
        get_otp_model_cls,
        get_profile_model_cls,
        get_tokens_for_user,
        response_for_client,
        get_setting,
    ):
        get_setting.side_effect = lambda key: {
            "OTP_MAX_ATTEMPTS": 5,
            "DEFAULT_PROFILE_ROLE": "USER",
        }[key]

        otp = SimpleNamespace(
            email="",
            phone="+525512345674",
            code="123456",
            attempts=0,
            is_used=False,
            save=MagicMock(),
        )
        otp_qs = MagicMock()
        otp_qs.filter.return_value = otp_qs
        otp_qs.order_by.return_value.first.return_value = otp

        otp_model = MagicMock()
        otp_model.objects.filter.return_value = otp_qs
        get_otp_model_cls.return_value = otp_model

        user_qs = MagicMock()
        user_qs.first.return_value = None
        user_cls.objects.filter.return_value = user_qs

        created_user = MagicMock()
        created_user.is_verified = False
        created_user.get_default_profile.return_value = MagicMock()
        user_cls.objects.create_user.return_value = created_user

        profile_model = MagicMock()
        get_profile_model_cls.return_value = profile_model

        get_tokens_for_user.return_value = {"access": "a", "refresh": "b"}
        response_for_client.return_value = {"ok": True}

        result = OtpService.verify_otp_code(
            {
                "phone": "+525512345674",
                "code": "123456",
                "client": "web",
            }
        )

        self.assertEqual(result, {"ok": True})
        user_cls.objects.create_user.assert_called_once_with(
            email="phone_525512345674@otp.local",
            phone="+525512345674",
            is_active=True,
        )



import logging

from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from jb_drf_auth.exceptions import SocialAuthError
from jb_drf_auth.serializers.social_auth import (
    SocialLinkSerializer,
    SocialLoginSerializer,
    SocialUnlinkSerializer,
)
from jb_drf_auth.services.social_auth import SocialAuthService
from jb_drf_auth.throttling import BasicLoginIPThrottle, BasicLoginIdentityThrottle

logger = logging.getLogger("jb_drf_auth.views.social_auth")


class SocialLoginView(APIView):
    permission_classes = []
    throttle_classes = [BasicLoginIPThrottle, BasicLoginIdentityThrottle]

    def post(self, request):
        serializer = SocialLoginSerializer(data=request.data, context={"request": request})
        serializer.is_valid(raise_exception=True)
        try:
            payload = serializer.save()
        except SocialAuthError as exc:
            logger.warning(
                "social_login_failed provider=%s code=%s status=%s client=%s",
                serializer.validated_data.get("provider"),
                exc.code,
                exc.status_code,
                serializer.validated_data.get("client"),
            )
            return Response({"detail": str(exc), "code": exc.code}, status=exc.status_code)
        logger.info(
            "social_login_success provider=%s client=%s user_created=%s linked_existing_user=%s",
            serializer.validated_data.get("provider"),
            serializer.validated_data.get("client"),
            payload.get("user_created"),
            payload.get("linked_existing_user"),
        )
        return Response(payload, status=status.HTTP_200_OK)


class SocialLinkView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        serializer = SocialLinkSerializer(data=request.data, context={"request": request})
        serializer.is_valid(raise_exception=True)
        try:
            payload = serializer.save()
        except SocialAuthError as exc:
            logger.warning(
                "social_link_failed provider=%s code=%s status=%s user_id=%s",
                serializer.validated_data.get("provider"),
                exc.code,
                exc.status_code,
                getattr(request.user, "id", None),
            )
            return Response({"detail": str(exc), "code": exc.code}, status=exc.status_code)
        logger.info(
            "social_link_success provider=%s created=%s user_id=%s",
            serializer.validated_data.get("provider"),
            payload.get("created"),
            getattr(request.user, "id", None),
        )
        return Response(payload, status=status.HTTP_200_OK)


class SocialPrecheckView(APIView):
    permission_classes = []
    throttle_classes = [BasicLoginIPThrottle, BasicLoginIdentityThrottle]

    def post(self, request):
        serializer = SocialLoginSerializer(data=request.data, context={"request": request})
        serializer.is_valid(raise_exception=True)
        try:
            payload = {
                "id_token": serializer.validated_data.get("id_token"),
                "access_token": serializer.validated_data.get("access_token"),
                "authorization_code": serializer.validated_data.get("authorization_code"),
                "redirect_uri": serializer.validated_data.get("redirect_uri"),
                "code_verifier": serializer.validated_data.get("code_verifier"),
                "client_id": serializer.validated_data.get("client_id"),
            }
            response_payload = SocialAuthService.precheck(
                provider_name=serializer.validated_data.get("provider"),
                payload=payload,
            )
        except SocialAuthError as exc:
            logger.warning(
                "social_precheck_failed provider=%s code=%s status=%s client=%s",
                serializer.validated_data.get("provider"),
                exc.code,
                exc.status_code,
                serializer.validated_data.get("client"),
            )
            return Response({"detail": str(exc), "code": exc.code}, status=exc.status_code)
        logger.info(
            "social_precheck_success provider=%s social_account_exists=%s linked_existing_user=%s would_create_user=%s",
            response_payload.get("provider"),
            response_payload.get("social_account_exists"),
            response_payload.get("linked_existing_user"),
            response_payload.get("would_create_user"),
        )
        return Response(response_payload, status=status.HTTP_200_OK)


class SocialUnlinkView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        serializer = SocialUnlinkSerializer(data=request.data, context={"request": request})
        serializer.is_valid(raise_exception=True)
        try:
            payload = serializer.save()
        except SocialAuthError as exc:
            logger.warning(
                "social_unlink_failed provider=%s code=%s status=%s user_id=%s",
                serializer.validated_data.get("provider"),
                exc.code,
                exc.status_code,
                getattr(request.user, "id", None),
            )
            return Response({"detail": str(exc), "code": exc.code}, status=exc.status_code)
        logger.info(
            "social_unlink_success provider=%s user_id=%s",
            serializer.validated_data.get("provider"),
            getattr(request.user, "id", None),
        )
        return Response(payload, status=status.HTTP_200_OK)



from django.conf import settings

DEFAULTS = {
    "PROFILE_MODEL": None,  # required: "accounts.Profile"
    "DEVICE_MODEL": None,  # required for mobile flows: "accounts.Device"
    "OTP_MODEL": None,  # required for otp flows: "accounts.OtpCode"
    "AUTHENTICATION_TYPE": "email",  # "email", "username", "both"
    "CLIENT_CHOICES": ("web", "mobile"),
    "AUTH_SINGLE_SESSION_ON_MOBILE": False,
    "MOBILE_NOTIFICATION_TOKEN_REQUIRED": False,
    "FRONTEND_URL": None,
    "DEFAULT_FROM_EMAIL": None,
    "TERMS_AND_CONDITIONS_REQUIRED": True,
    "EMAIL_PROVIDER": "jb_drf_auth.providers.django_email.DjangoEmailProvider",
    "EMAIL_LOG_MODEL": None,  # required for email flows: "authentication.EmailLog"
    "EMAIL_TEMPLATES": None,
    "ADMIN_BOOTSTRAP_TOKEN": None,
    "OTP_LENGTH": 6,
    "OTP_TTL_SECONDS": 300,
    "OTP_MAX_ATTEMPTS": 5,
    "OTP_RESEND_COOLDOWN_SECONDS": 60,
    "OTP_PHONE_EMAIL_DOMAIN": "phone.local",
    "PROFILE_ROLE_CHOICES": (
        ("USER", "Usuario"),
        ("COMMERCE", "Comercio"),
        ("ADMIN", "Admin"),
    ),
    "PROFILE_GENDER_CHOICES": (
        ("MALE", "Masculino"),
        ("FEMALE", "Femenino"),
        ("OTHER", "Otro"),
        ("PREFER_NOT_TO_SAY", "Prefiero no decirlo"),
    ),
    "DEFAULT_PROFILE_ROLE": "USER",
    "PROFILE_ID_CLAIM": "profile_id",
    "PROFILE_PICTURE_UPLOAD_TO": "uploads/users/profile-pictures",
    "PROFILE_PICTURE_OPTIMIZE": True,
    "PROFILE_PICTURE_MAX_BYTES": 1024 * 1024,
    "PROFILE_PICTURE_MAX_WIDTH": 1080,
    "PROFILE_PICTURE_MAX_HEIGHT": 1080,
    "PROFILE_PICTURE_JPEG_QUALITY": 85,
    "PROFILE_PICTURE_MIN_JPEG_QUALITY": 65,
    "PERSON_PICTURE_UPLOAD_TO": None,
    "PERSON_ID_DOCUMENTS_UPLOAD_TO": "uploads/people/id-documents",
    "SMS_PROVIDER": "jb_drf_auth.providers.aws_sns.AwsSnsSmsProvider",
    "SMS_SENDER_ID": None,
    "SMS_TYPE": "Transactional",
    "SMS_OTP_MESSAGE": "Tu codigo es {code}. Expira en {minutes} minutos.",
    "TWILIO_ACCOUNT_SID": None,
    "TWILIO_AUTH_TOKEN": None,
    "TWILIO_FROM_NUMBER": None,
    "TWILIO_MESSAGING_SERVICE_SID": None,
    "SMS_LOG_MODEL": None,  # optional: "accounts.SmsLog"
    "PHONE_DEFAULT_COUNTRY_CODE": None,
    "PHONE_MIN_LENGTH": 10,
    "PHONE_MAX_LENGTH": 15,
    "THROTTLE_ENABLED": True,
    "THROTTLE_RATES": {
        "LOGIN_IP": "20/min",
        "LOGIN_IDENTITY": "10/min",
        "REGISTER_IP": "10/hour",
        "REGISTER_IDENTITY": "5/hour",
        "OTP_REQUEST_IP": "20/hour",
        "OTP_REQUEST_IDENTITY": "30/hour",
        "OTP_VERIFY_IP": "30/hour",
        "OTP_VERIFY_IDENTITY": "60/hour",
        "PASSWORD_RESET_REQUEST_IP": "15/hour",
        "PASSWORD_RESET_REQUEST_IDENTITY": "5/hour",
        "PASSWORD_RESET_CONFIRM_IP": "30/hour",
        "EMAIL_CONFIRMATION_IP": "30/hour",
        "EMAIL_CONFIRMATION_RESEND_IP": "10/hour",
        "EMAIL_CONFIRMATION_RESEND_IDENTITY": "5/hour",
    },
    "SOCIAL_ACCOUNT_MODEL": None,  # required for social login: "authentication.SocialAccount"
    "SOCIAL": {
        "DEBUG_ERRORS": False,
        "AUTO_CREATE_USER": True,
        "LINK_BY_EMAIL": True,
        "REQUIRE_VERIFIED_EMAIL": True,
        "SYNC_PICTURE_ON_LOGIN": True,
        "PICTURE_DOWNLOAD_TIMEOUT_SECONDS": 5,
        "PICTURE_MAX_BYTES": 5 * 1024 * 1024,
        "PICTURE_ALLOWED_CONTENT_TYPES": ("image/jpeg", "image/png", "image/webp"),
        "PROVIDERS": {
            "google": {
                "CLASS": "jb_drf_auth.providers.google_oidc.GoogleOidcProvider",
                "CLIENT_IDS": (),
                "CLIENT_SECRET": None,
                "ISSUER": "https://accounts.google.com",
                "JWKS_URL": "https://www.googleapis.com/oauth2/v3/certs",
                "TOKEN_URL": "https://oauth2.googleapis.com/token",
            },
            "apple": {
                "CLASS": "jb_drf_auth.providers.apple_oidc.AppleOidcProvider",
                "CLIENT_IDS": (),
                "CLIENT_SECRET": None,
                "ISSUER": "https://appleid.apple.com",
                "JWKS_URL": "https://appleid.apple.com/auth/keys",
                "TOKEN_URL": "https://appleid.apple.com/auth/token",
            },
            "facebook": {
                "CLASS": "jb_drf_auth.providers.facebook_oauth.FacebookOAuthProvider",
                "APP_ID": None,
                "APP_SECRET": None,
                "GRAPH_API_VERSION": "v21.0",
                "ASSUME_EMAIL_VERIFIED": True,
            },
        },
    },
}

PREFIX = "JB_DRF_AUTH_"
ROOT_SETTING = "JB_DRF_AUTH"


def get_setting(name: str):
    root = getattr(settings, ROOT_SETTING, None)
    if isinstance(root, dict) and name in root:
        return root[name]
    prefixed_name = f"{PREFIX}{name}"
    if hasattr(settings, prefixed_name):
        return getattr(settings, prefixed_name)
    if hasattr(settings, name):
        return getattr(settings, name)
    return DEFAULTS.get(name)


def get_social_settings():
    defaults = DEFAULTS["SOCIAL"]
    configured = get_setting("SOCIAL")
    if not isinstance(configured, dict):
        return defaults

    merged = {**defaults, **configured}
    default_providers = defaults.get("PROVIDERS", {})
    configured_providers = configured.get("PROVIDERS", {})

    providers = {}
    if isinstance(default_providers, dict):
        for provider_name, provider_cfg in default_providers.items():
            providers[provider_name] = (
                dict(provider_cfg) if isinstance(provider_cfg, dict) else provider_cfg
            )

    if isinstance(configured_providers, dict):
        for provider_name, provider_cfg in configured_providers.items():
            base_cfg = providers.get(provider_name, {})
            if isinstance(base_cfg, dict) and isinstance(provider_cfg, dict):
                providers[provider_name] = {**base_cfg, **provider_cfg}
            else:
                providers[provider_name] = provider_cfg

    # Normalize providers and remove empty client IDs.
    for provider_name, provider_cfg in list(providers.items()):
        if not isinstance(provider_cfg, dict):
            continue
        client_ids = provider_cfg.get("CLIENT_IDS")
        if isinstance(client_ids, str):
            client_ids = (client_ids,)
        normalized_client_ids = []
        if isinstance(client_ids, (tuple, list)):
            normalized_client_ids.extend(client_ids)
        # Allow discrete keys instead of a tuple in integrator settings.
        normalized_client_ids.extend(
            [
                provider_cfg.get("CLIENT_ID"),
                provider_cfg.get("CLIENT_ID_WEB"),
                provider_cfg.get("CLIENT_ID_IOS"),
                provider_cfg.get("CLIENT_ID_ANDROID"),
            ]
        )
        cleaned = []
        for value in normalized_client_ids:
            if not isinstance(value, str):
                continue
            value = value.strip()
            if value and value not in cleaned:
                cleaned.append(value)
        providers[provider_name]["CLIENT_IDS"] = tuple(cleaned)

    merged["PROVIDERS"] = providers
    return merged
